"use strict";
/**

Copyright 2019 Forestry.io Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_tinacms_1 = require("react-tinacms");
var react_1 = require("react");
function useJsonForm(jsonNode, formOptions) {
    if (formOptions === void 0) { formOptions = {}; }
    if (!jsonNode || process.env.NODE_ENV === 'production') {
        return [{}, null];
    }
    validateJsonNode(jsonNode);
    var cms = react_tinacms_1.useCMS();
    var label = jsonNode.fileRelativePath;
    var id = jsonNode.fileRelativePath;
    /**
     * The state of the JsonForm, generated from the contents of the
     * Json file currently on disk. This state will contain any
     * un-committed changes in the Json file.
     */
    var valuesOnDisk = react_1.useMemo(function () { return ({
        jsonNode: jsonNode,
        rawJson: JSON.parse(jsonNode.rawJson),
    }); }, [jsonNode]);
    /**
     * The state of the JsonForm, generated from the contents of the
     * Json file at the HEAD of this git branch.
     */
    var _a = react_1.useState(), valuesInGit = _a[0], setValuesInGit = _a[1];
    react_1.useEffect(function () {
        cms.api.git
            .show(id) // Load the contents of this file at HEAD
            .then(function (git) {
            // Parse the JSON into a JsonForm data structure and store it in state.
            var rawJson = JSON.parse(git.content);
            setValuesInGit({ jsonNode: jsonNode, rawJson: rawJson });
        })
            .catch(function (e) {
            console.log('FAILED', e);
        });
    }, [id]);
    var fields = formOptions.fields || generateFields(valuesOnDisk.rawJson);
    // TODO: This may not be necessary.
    fields.push({ name: 'jsonNode', component: null });
    var _b = react_tinacms_1.useCMSForm(__assign({ id: id,
        label: label, initialValues: valuesInGit, fields: fields,
        onSubmit: function (data) {
            return cms.api.git.onSubmit({
                files: [data.fileRelativePath],
                message: data.__commit_message || 'Tina commit',
                name: data.__commit_name,
                email: data.__commit_email,
            });
        },
        reset: function () {
            return cms.api.git.reset({ files: [id] });
        } }, formOptions), 
    // The Form will be updated if these values change.
    { values: valuesOnDisk, label: label, fields: fields }), values = _b[0], form = _b[1];
    var writeToDisk = react_1.useCallback(function (formState) {
        var _a = formState.values.rawJson, fileRelativePath = _a.fileRelativePath, rawJson = _a.rawJson, data = __rest(_a, ["fileRelativePath", "rawJson"]);
        cms.api.git.onChange({
            fileRelativePath: formState.values.jsonNode.fileRelativePath,
            content: JSON.stringify(data, null, 2),
        });
    }, []);
    react_tinacms_1.useWatchFormValues(form, writeToDisk);
    return [jsonNode, form];
}
exports.useJsonForm = useJsonForm;
function generateFields(post) {
    return Object.keys(post).map(function (key) { return ({
        component: 'text',
        name: "rawJson." + key,
    }); });
}
function JsonForm(_a) {
    var data = _a.data, render = _a.render, options = __rest(_a, ["data", "render"]);
    var _b = useJsonForm(data, options), currentData = _b[0], form = _b[1];
    return render({ form: form, data: currentData });
}
exports.JsonForm = JsonForm;
function validateJsonNode(jsonNode) {
    // TODO
}
